@@ -28,10 +28,11 @@ class Connection(object):
         )
 
         # we only support one cursor per connection
-        self._cursor = Cursor(self, None)
+        self.options.setdefault('unicode_error', None)
+        self._cursor = Cursor(self, None, unicode_error=self.options['unicode_error'])
         self.options.setdefault('port', 5433)
         self.options.setdefault('read_timeout', 600)
-        self.boot_connection()
+        self.startup_connection()
 
     def __enter__(self):
         return self
@@ -99,24 +100,31 @@ class Connection(object):
         if self.socket is not None:
             return self.socket
 
-        if self.options.get('ssl'):
-            # SSL
-            raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            raw_socket.connect((self.options['host'], self.options['port']))
+        host = self.options.get('host')
+        port = self.options.get('port')
+        raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        raw_socket.connect((host, port))
+
+        ssl_options = self.options.get('ssl')
+        if ssl_options is not None and not ssl_options:
+            from ssl import CertificateError, SSLError
             raw_socket.sendall(messages.SslRequest().to_bytes())
             response = raw_socket.recv(1)
             if response == 'S':
-                # May want to add certs to this
-                raw_socket = ssl.wrap_socket(raw_socket)
+                try:
+                    if isinstance(ssl_options, ssl.SSLContext):
+                        raw_socket = ssl_options.wrap_socket(raw_socket, server_hostname=host)
+                    else:
+                        raw_socket = ssl.wrap_socket(raw_socket)
+                except CertificateError, e:
+                    raise errors.ConnectionError('SSL: ' + e.message)
+                except SSLError, e:
+                    raise errors.ConnectionError('SSL: ' + e.reason)
             else:
                 raise SSLNotSupported("SSL requested but not supported by server")
-        else:
-            # Non-SSL
-            raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            raw_socket.connect((self.options['host'], self.options['port']))
 
         self.socket = raw_socket
-        return raw_socket
+        return self.socket
 
     def ssl(self):
         return self.socket is not None and isinstance(ssl.SSLSocket, self.socket)
@@ -152,7 +160,10 @@ class Connection(object):
 
         except Exception, e:
             self.close_socket()
-            raise errors.ConnectionError(e.message)
+            if e.message == 'unsupported authentication method: 9':
+                raise errors.ConnectionError('Error during authentication. Your password might be expired.')
+            else:
+                raise errors.ConnectionError(e.message)
 
     def close_socket(self):
         try:
@@ -163,32 +174,21 @@ class Connection(object):
 
     def reset_connection(self):
         self.close()
-        self.boot_connection()
-
-    def boot_connection(self):
         self.startup_connection()
-        self.initialize_connection()
 
     def read_message(self):
         try:
-            ready = select.select([self._socket()], [], [], self.options['read_timeout'])
-            if len(ready[0]) > 0:
-                type = self.read_bytes(1)
-                size = unpack('!I', self.read_bytes(4))[0]
+            type = self.read_bytes(1)
+            size = unpack('!I', self.read_bytes(4))[0]
 
-                if size < 4:
-                    raise errors.MessageError(
-                        "Bad message size: {0}".format(size)
-                    )
-                message = BackendMessage.factory(type, self.read_bytes(size - 4))
-                logger.debug('<= %s', message)
-                return message
-            else:
-                self.close()
-                raise errors.TimedOutError("Connection timed out")
-        except errors.TimedOutError:
-            raise
-        except Exception as e:
+            if size < 4:
+                raise errors.MessageError(
+                    "Bad message size: {0}".format(size)
+                )
+            message = BackendMessage.factory(type, self.read_bytes(size - 4))
+            logger.debug('<= %s', message)
+            return message
+        except (SystemError, IOError) as e:
             self.close_socket()
             raise errors.ConnectionError(e.message)
 
@@ -263,12 +263,3 @@ class Connection(object):
 
             if isinstance(message, messages.ReadyForQuery):
                 break
-
-    def initialize_connection(self):
-        if self.options.get('search_path') is not None:
-            self.query("SET SEARCH_PATH TO {0}".format(self.options['search_path']))
-        if self.options.get('role') is not None:
-            self.query("SET ROLE {0}".format(self.options['role']))
-
-# if self.options.get('interruptable'):
-#            self.session_id = self.query("SELECT session_id FROM v_monitor.current_session").the_value()
